@@ -1,9 +1,10 @@
 from collections import namedtuple
+from enum import Enum
 import os
 
-# A list of metadata fields which dandi extracts from .nwb files.
-# Additional fields (such as `number_of_*`) might be added by the
-# get_metadata`
+#: A list of metadata fields which dandi extracts from .nwb files.
+#: Additional fields (such as ``number_of_*``) might be added by
+#: `get_metadata()`
 metadata_nwb_file_fields = (
     "experiment_description",
     "experimenter",
@@ -64,9 +65,27 @@ metadata_dandiset_fields = (
 
 metadata_all_fields = metadata_nwb_fields + metadata_dandiset_fields
 
+#: Regular expression for a valid Dandiset identifier.  This regex is not
+#: anchored.
+DANDISET_ID_REGEX = r"[0-9]{6}"
+
+#: Regular expression for a valid published (i.e., non-draft) Dandiset version
+#: identifier.  This regex is not anchored.
+PUBLISHED_VERSION_REGEX = r"[0-9]+\.[0-9]+\.[0-9]+"
+
+#: Regular expression for a valid Dandiset version identifier.  This regex is
+#: not anchored.
+VERSION_REGEX = fr"(?:{PUBLISHED_VERSION_REGEX}|draft)"
+
+
+class EmbargoStatus(Enum):
+    OPEN = "OPEN"
+    UNEMBARGOING = "UNEMBARGOING"
+    EMBARGOED = "EMBARGOED"
+
+
 dandiset_metadata_file = "dandiset.yaml"
-dandiset_identifier_regex = "^[0-9]{6}$"
-VERSION_REGEX = r"(?:[.0-9]{5,}|draft)"
+dandiset_identifier_regex = f"^{DANDISET_ID_REGEX}$"
 
 DandiInstance = namedtuple("DandiInstance", ("gui", "redirector", "api"))
 
@@ -109,12 +128,17 @@ file_operation_modes = [
     "auto",
 ]
 
-#
+
 # Download (upload?) specific constants
-#
-# Chunk size when iterating a download (and upload) body. Taken from girder-cli
-# TODO: should we make them smaller for download than for upload?
-# ATM used only in download
+
+#: Chunk size when iterating a download (and upload) body. Taken from girder-cli
+#: TODO: should we make them smaller for download than for upload?
+#: ATM used only in download
 MAX_CHUNK_SIZE = int(os.environ.get("DANDI_MAX_CHUNK_SIZE", 1024 * 1024 * 8))  # 64
 
+#: The identifier for draft Dandiset versions
 DRAFT = "draft"
+
+#: HTTP response status codes that should always be retried (until we run out
+#: of retries)
+RETRY_STATUSES = (500, 502, 503, 504)
