@@ -11,10 +11,12 @@ import time
 import humanize
 import requests
 
-from .consts import dandiset_metadata_file
+from . import get_logger
+from .consts import RETRY_STATUSES, dandiset_metadata_file
 from .dandiarchive import DandisetURL, MultiAssetURL, SingleAssetURL, parse_dandi_url
 from .dandiset import Dandiset
-from . import get_logger
+from .exceptions import NotFoundError
+from .support.digests import get_digest
 from .support.pyout import naturalsize
 from .utils import (
     abbrev_prompt,
@@ -23,7 +25,9 @@ from .utils import (
     flattened,
     is_same_time,
     on_windows,
+    path_is_subpath,
     pluralize,
+    yaml_load,
 )
 
 lgr = get_logger()
@@ -109,9 +113,8 @@ def download(
         raise ValueError(format)
 
     if sync and not isinstance(parsed_url, SingleAssetURL):
-        client = parsed_url.get_client()
-        with client.session():
-            asset_paths = {asset["path"] for asset in parsed_url.get_assets(client)}
+        with parsed_url.get_client() as client:
+            asset_paths = {asset.path for asset in parsed_url.get_assets(client)}
         if isinstance(parsed_url, DandisetURL):
             prefix = os.curdir
             download_dir = output_path
@@ -177,7 +180,7 @@ def download_generator(
 
     """
 
-    with parsed_url.navigate() as (client, dandiset, assets):
+    with parsed_url.navigate(strict=True) as (client, dandiset, assets):
         if assets_it:
             assets_it.gen = assets
             assets = assets_it
@@ -194,21 +197,7 @@ def download_generator(
             return
 
         for asset in assets:
-            down_args = (
-                dandiset["dandiset"]["identifier"],
-                dandiset["version"],
-                asset["asset_id"],
-            )
-            metadata = asset["metadata"]
-            d = metadata.get("digest", {})
-            if "dandi:dandi-etag" in d:
-                digests = {"dandi-etag": d["dandi:dandi-etag"]}
-            else:
-                raise RuntimeError(
-                    f"dandi-etag not available for asset. Known digests: {d}"
-                )
-
-            path = asset["path"].lstrip("/")  # make into relative path
+            path = asset.path.lstrip("/")  # make into relative path
             path = op.normpath(path)
             if not isinstance(parsed_url, DandisetURL):
                 if isinstance(parsed_url, MultiAssetURL):
@@ -224,25 +213,43 @@ def download_generator(
                     )
             download_path = op.join(output_path, path)
 
-            downloader = client.get_download_file_iter(*down_args)
+            try:
+                metadata = asset.get_raw_metadata()
+            except NotFoundError as e:
+                yield {"path": path, "status": "error", "message": str(e)}
+                continue
+            d = metadata.get("digest", {})
+            if "dandi:dandi-etag" in d:
+                digests = {"dandi-etag": d["dandi:dandi-etag"]}
+            else:
+                raise RuntimeError(
+                    f"dandi-etag not available for asset. Known digests: {d}"
+                )
+            try:
+                digests["sha256"] = d["dandi:sha2-256"]
+            except KeyError:
+                pass
+
+            downloader = asset.get_download_file_iter()
 
             try:
-                mtime = asset["metadata"]["blobDateModified"]
+                mtime = metadata["blobDateModified"]
             except KeyError:
                 mtime = None
             if mtime is None:
                 lgr.warning(
                     "Asset %s is missing blobDateModified metadata field",
-                    asset["path"],
+                    asset.path,
                 )
-                mtime = asset.get("modified")
+                mtime = asset.modified
 
             _download_generator = _download_file(
                 downloader,
                 download_path,
+                toplevel_path=output_path,
                 # size and modified generally should be there but better to redownload
                 # than to crash
-                size=asset.get("size"),
+                size=asset.size,
                 mtime=mtime,
                 existing=existing,
                 digests=digests,
@@ -276,14 +283,7 @@ class ItemsSummary:
         if not self.files:
             self.t0 = time.time()
         self.files += 1
-        size = rec.get("size")
-        if size is not None:
-            self.size += size
-        elif rec.get("path", "") == "dandiset.yaml":
-            # again -- so special. TODO: make it a proper file
-            pass
-        else:
-            self.has_unknown_sizes = True
+        self.size += rec.size
         return self
 
 
@@ -378,8 +378,8 @@ def _skip_file(msg):
     return {"status": "skipped", "message": str(msg)}
 
 
-def _populate_dandiset_yaml(dandiset_path, dandiset_data, existing):
-    metadata = dandiset_data.get("metadata")
+def _populate_dandiset_yaml(dandiset_path, dandiset, existing):
+    metadata = dandiset.get_raw_metadata()
     if not metadata:
         lgr.warning(
             "Got completely empty metadata record for dandiset, not producing dandiset.yaml"
@@ -388,20 +388,29 @@ def _populate_dandiset_yaml(dandiset_path, dandiset_data, existing):
     dandiset_yaml = op.join(dandiset_path, dandiset_metadata_file)
     yield {"message": "updating"}
     lgr.debug("Updating %s from obtained dandiset metadata", dandiset_metadata_file)
-    mtime = ensure_datetime(dandiset_data["modified"])
+    mtime = dandiset.modified
     if op.lexists(dandiset_yaml):
+        with open(dandiset_yaml) as fp:
+            if yaml_load(fp, typ="safe") == metadata:
+                yield _skip_file("no change")
+                return
         if existing == "error":
-            raise FileExistsError(dandiset_yaml)
+            yield {"status": "error", "message": "already exists"}
+            return
+        elif existing == "refresh" and op.lexists(
+            op.join(dandiset_path, ".git", "annex")
+        ):
+            raise RuntimeError("Not refreshing path in git annex repository")
         elif existing == "skip" or (
             existing == "refresh"
             and os.lstat(dandiset_yaml).st_mtime >= mtime.timestamp()
         ):
             yield _skip_file("already exists")
             return
-    dandiset = Dandiset(dandiset_path, allow_empty=True)
-    dandiset.path_obj.mkdir(exist_ok=True)  # exist_ok in case of parallel race
-    old_metadata = dandiset.metadata
-    dandiset.update_metadata(metadata)
+    ds = Dandiset(dandiset_path, allow_empty=True)
+    ds.path_obj.mkdir(exist_ok=True)  # exist_ok in case of parallel race
+    old_metadata = ds.metadata
+    ds.update_metadata(metadata)
     os.utime(dandiset_yaml, (time.time(), mtime.timestamp()))
     yield {
         "status": "done",
@@ -410,7 +419,13 @@ def _populate_dandiset_yaml(dandiset_path, dandiset_data, existing):
 
 
 def _download_file(
-    downloader, path, size=None, mtime=None, existing="error", digests=None
+    downloader,
+    path,
+    toplevel_path,
+    size=None,
+    mtime=None,
+    existing="error",
+    digests=None,
 ):
     """Common logic for downloading a single file
 
@@ -434,6 +449,7 @@ def _download_file(
     """
     if op.lexists(path):
         block = f"File {path!r} already exists"
+        annex_path = op.join(toplevel_path, ".git", "annex")
         if existing == "error":
             raise FileExistsError(block)
         elif existing == "skip":
@@ -441,7 +457,39 @@ def _download_file(
             return
         elif existing == "overwrite":
             pass
+        elif existing == "overwrite-different":
+            realpath = op.realpath(path)
+            key_parts = op.basename(realpath).split("-")
+            if size is not None and os.stat(realpath).st_size != size:
+                lgr.debug(
+                    "Size of %s does not match size on server; redownloading", path
+                )
+            elif (
+                op.lexists(annex_path)
+                and op.islink(path)
+                and path_is_subpath(realpath, op.abspath(annex_path))
+                and key_parts[0] == "SHA256E"
+                and digests
+                and "sha256" in digests
+            ):
+                if key_parts[-1].partition(".")[0] == digests["sha256"]:
+                    yield _skip_file("already exists")
+                    return
+                else:
+                    lgr.debug(
+                        "%s is in git-annex, and hash does not match hash on server; redownloading",
+                        path,
+                    )
+            elif get_digest(path, "dandi-etag") == digests["dandi-etag"]:
+                yield _skip_file("already exists")
+                return
+            else:
+                lgr.debug(
+                    "Etag of %s does not match etag on server; redownloading", path
+                )
         elif existing == "refresh":
+            if op.lexists(annex_path):
+                raise RuntimeError("Not refreshing path in git annex repository")
             if mtime is None:
                 lgr.warning(
                     f"{path!r} - no mtime or ctime in the record, redownloading"
@@ -475,7 +523,7 @@ def _download_file(
         # TODO: reuse that sorting based on speed
         for algo, digest in digests.items():
             if algo == "dandi-etag":
-                from .core.digests.dandietag import ETagHashlike
+                from dandischema.digests.dandietag import ETagHashlike
 
                 digester = lambda: ETagHashlike(size)  # noqa: E731
             else:
@@ -487,6 +535,7 @@ def _download_file(
 
     # TODO: how do we discover the total size????
     # TODO: do not do it in-place, but rather into some "hidden" file
+    resuming = False
     for attempt in range(3):
         try:
             if digester:
@@ -495,6 +544,7 @@ def _download_file(
             # I wonder if we could make writing async with downloader
             with DownloadDirectory(path, digests) as dldir:
                 downloaded = dldir.offset
+                resuming = downloaded > 0
                 if size is not None and downloaded == size:
                     # Exit early when downloaded == size, as making a Range
                     # request in such a case results in a 416 error from S3.
@@ -527,7 +577,7 @@ def _download_file(
             if attempt >= 2 or exc.response.status_code not in (
                 400,  # Bad Request, but happened with gider:
                 # https://github.com/dandi/dandi-cli/issues/87
-                503,  # Service Unavailable
+                *RETRY_STATUSES,
             ):
                 lgr.debug("Download failed: %s", exc)
                 yield {"status": "error", "message": str(exc)}
@@ -542,7 +592,7 @@ def _download_file(
             )
             time.sleep(random.random() * 5)
 
-    if downloaded_digest:
+    if downloaded_digest and not resuming:
         downloaded_digest = downloaded_digest.hexdigest()  # we care only about hex
         if digest != downloaded_digest:
             msg = f"{algo}: downloaded {downloaded_digest} != {digest}"
