@@ -1,18 +1,22 @@
 import json
 import os
 import os.path as op
-from pathlib import Path
+import re
 from shutil import rmtree
 
 import pytest
+import responses
 
-from ..consts import DRAFT
-from ..download import download
-from ..utils import find_files
+from .skip import mark
+from ..consts import DRAFT, dandiset_metadata_file
+from ..dandiarchive import DandisetURL
+from ..download import download, download_generator
+from ..utils import list_paths
 
 
 # both urls point to 000027 (lean test dataset), and both draft and "released"
 # version have only a single file ATM
+@mark.skipif_no_network
 @pytest.mark.parametrize(
     "url",
     [  # Should go through API
@@ -48,6 +52,7 @@ def test_download_000027(url, tmpdir):
     download(url, tmpdir, existing="refresh")  # TODO: check that skipped (the same)
 
 
+@mark.skipif_no_network
 @pytest.mark.parametrize(
     "url",
     [  # Should go through API
@@ -64,6 +69,7 @@ def test_download_000027_metadata_only(url, tmpdir):
     assert sorted(downloads) == ["dandiset.yaml"]
 
 
+@mark.skipif_no_network
 @pytest.mark.parametrize(
     "url",
     [  # Should go through API
@@ -80,6 +86,7 @@ def test_download_000027_assets_only(url, tmpdir):
     assert sorted(downloads) == ["sub-RAT123", op.join("sub-RAT123", "sub-RAT123.nwb")]
 
 
+@mark.skipif_no_network
 @pytest.mark.parametrize("resizer", [lambda sz: 0, lambda sz: sz // 2, lambda sz: sz])
 @pytest.mark.parametrize("version", ["0.210831.2033", DRAFT])
 def test_download_000027_resume(tmp_path, resizer, version):
@@ -110,15 +117,17 @@ def test_download_000027_resume(tmp_path, resizer, version):
     assert digester(str(nwb)) == digests
 
 
-def test_download_newest_version(local_dandi_api, text_dandiset, tmp_path):
+def test_download_newest_version(text_dandiset, tmp_path):
+    dandiset = text_dandiset["dandiset"]
     dandiset_id = text_dandiset["dandiset_id"]
-    download(f"{local_dandi_api['instance'].api}/dandisets/{dandiset_id}", tmp_path)
+    download(dandiset.api_url, tmp_path)
     assert (tmp_path / dandiset_id / "file.txt").read_text() == "This is test text.\n"
-    text_dandiset["dandiset"].publish()
+    dandiset.wait_until_valid()
+    dandiset.publish()
     (text_dandiset["dspath"] / "file.txt").write_text("This is different text.\n")
     text_dandiset["reupload"]()
     rmtree(tmp_path / dandiset_id)
-    download(f"{local_dandi_api['instance'].api}/dandisets/{dandiset_id}", tmp_path)
+    download(dandiset.api_url, tmp_path)
     assert (tmp_path / dandiset_id / "file.txt").read_text() == "This is test text.\n"
 
 
@@ -127,7 +136,7 @@ def test_download_folder(local_dandi_api, text_dandiset, tmp_path):
     download(
         f"dandi://{local_dandi_api['instance_id']}/{dandiset_id}/subdir2/", tmp_path
     )
-    assert sorted(map(Path, find_files(r".*", paths=[tmp_path], dirs=True))) == [
+    assert list_paths(tmp_path, dirs=True) == [
         tmp_path / "subdir2",
         tmp_path / "subdir2" / "banana.txt",
         tmp_path / "subdir2" / "coconut.txt",
@@ -142,35 +151,21 @@ def test_download_item(local_dandi_api, text_dandiset, tmp_path):
         f"dandi://{local_dandi_api['instance_id']}/{dandiset_id}/subdir2/coconut.txt",
         tmp_path,
     )
-    assert list(map(Path, find_files(r".*", paths=[tmp_path], dirs=True))) == [
-        tmp_path / "coconut.txt"
-    ]
+    assert list_paths(tmp_path, dirs=True) == [tmp_path / "coconut.txt"]
     assert (tmp_path / "coconut.txt").read_text() == "Coconut\n"
 
 
-def test_download_asset_id(local_dandi_api, text_dandiset, tmp_path):
-    dandiset_id = text_dandiset["dandiset_id"]
+def test_download_asset_id(text_dandiset, tmp_path):
     asset = text_dandiset["dandiset"].get_asset_by_path("subdir2/coconut.txt")
-    download(
-        f"{local_dandi_api['instance'].api}/dandisets/{dandiset_id}/versions"
-        f"/draft/assets/{asset.identifier}/download/",
-        tmp_path,
-    )
-    assert list(map(Path, find_files(r".*", paths=[tmp_path], dirs=True))) == [
-        tmp_path / "coconut.txt"
-    ]
+    download(asset.download_url, tmp_path)
+    assert list_paths(tmp_path, dirs=True) == [tmp_path / "coconut.txt"]
     assert (tmp_path / "coconut.txt").read_text() == "Coconut\n"
 
 
-def test_download_asset_id_only(local_dandi_api, text_dandiset, tmp_path):
+def test_download_asset_id_only(text_dandiset, tmp_path):
     asset = text_dandiset["dandiset"].get_asset_by_path("subdir2/coconut.txt")
-    download(
-        f"{local_dandi_api['instance'].api}/assets/{asset.identifier}/download/",
-        tmp_path,
-    )
-    assert list(map(Path, find_files(r".*", paths=[tmp_path], dirs=True))) == [
-        tmp_path / "coconut.txt"
-    ]
+    download(asset.base_download_url, tmp_path)
+    assert list_paths(tmp_path, dirs=True) == [tmp_path / "coconut.txt"]
     assert (tmp_path / "coconut.txt").read_text() == "Coconut\n"
 
 
@@ -227,3 +222,47 @@ def test_download_sync_list(capsys, local_dandi_api, mocker, text_dandiset, tmp_
         mocker.call("Delete 1 local asset? ([y]es/[n]o/[l]ist): "),
     ]
     assert capsys.readouterr().out.splitlines()[-1] == str(dspath / "file.txt")
+
+
+@responses.activate
+def test_download_no_blobDateModified(text_dandiset, tmp_path):
+    # Regression test for #806
+    responses.add_passthru(re.compile("^http"))
+    dandiset = text_dandiset["dandiset"]
+    asset = dandiset.get_asset_by_path("file.txt")
+    metadata = asset.get_raw_metadata()
+    del metadata["blobDateModified"]
+    responses.add(responses.GET, asset.api_url, json=metadata)
+    download(dandiset.api_url, tmp_path)
+
+
+@responses.activate
+def test_download_metadata404(text_dandiset, tmp_path):
+    responses.add_passthru(re.compile("^http"))
+    asset = text_dandiset["dandiset"].get_asset_by_path("subdir1/apple.txt")
+    responses.add(responses.GET, asset.api_url, status=404)
+    statuses = list(
+        download_generator(
+            DandisetURL(
+                api_url=text_dandiset["client"].api_url,
+                dandiset_id=text_dandiset["dandiset"].identifier,
+                version_id=text_dandiset["dandiset"].version_id,
+            ),
+            tmp_path,
+        )
+    )
+    errors = [s for s in statuses if s.get("status") == "error"]
+    assert errors == [
+        {
+            "path": "subdir1/apple.txt",
+            "status": "error",
+            "message": f"No such asset: {asset}",
+        }
+    ]
+    assert list_paths(tmp_path, dirs=True) == [
+        tmp_path / dandiset_metadata_file,
+        tmp_path / "file.txt",
+        tmp_path / "subdir2",
+        tmp_path / "subdir2" / "banana.txt",
+        tmp_path / "subdir2" / "coconut.txt",
+    ]
