@@ -9,7 +9,6 @@ except ImportError:
 import inspect
 import io
 import itertools
-import logging
 from mimetypes import guess_type
 import os
 import os.path as op
@@ -20,19 +19,18 @@ import shutil
 import subprocess
 import sys
 import types
-from typing import Optional, Union
+from typing import List, Optional, Union
 
 import dateutil.parser
 import requests
 import ruamel.yaml
 from semantic_version import Version
-import tenacity
 
 #
 # Additional handlers
 #
 from . import __version__, get_logger
-from .consts import dandi_instance, known_instances, known_instances_rev
+from .consts import DandiInstance, known_instances, known_instances_rev
 from .exceptions import BadCliVersionError, CliVersionTooOldError
 
 lgr = get_logger()
@@ -213,39 +211,6 @@ def flattened(it):
     return list(flatten(it))
 
 
-def remap_dict(rec, revmapping):
-    """Remap nested dicts according to mapping
-
-    Parameters
-    ----------
-    revmapping: dict
-      (to, from)
-
-    TODO: document and test more
-    """
-    out = {}
-
-    def split(path):
-        # map path from key.subkey if given in a string (not tuple) form
-        return path.split(".") if isinstance(path, str) else path
-
-    for to, from_ in revmapping.items():
-        in_v = rec
-        for p in split(from_):
-            if p not in in_v:
-                continue  # it is not there -- cannot map
-            in_v = in_v[p]
-
-        # and now set
-        out_v = out
-        t_split = split(to)
-        for p in t_split[:-1]:
-            out_v[p] = out_v.get(p, {})  # container for the next field
-            out_v = out_v[p]
-        out_v[t_split[-1]] = in_v  # and the last one gets the in_v
-    return out
-
-
 #
 # Paths and files
 #
@@ -281,11 +246,11 @@ def find_files(
 
     Parameters
     ----------
-    regex: basestring
+    regex: string
       Regex to search target files. Is not applied to filter out directories
-    paths: basestring or list, optional
+    paths: string or list, optional
       Directories or files to search among (directories are searched recursively)
-    exclude: basestring, optional
+    exclude: string, optional
       Matches to exclude
     exclude_vcs:
       If True, excludes commonly known VCS subdirectories.  If string, used
@@ -359,6 +324,10 @@ def find_files(
                     yield path
 
 
+def list_paths(dirpath: Union[str, Path], dirs: bool = False) -> List[Path]:
+    return sorted(map(Path, find_files(r".*", [dirpath], dirs=dirs)))
+
+
 _cp_supports_reflink = None
 
 
@@ -602,7 +571,7 @@ def get_instance(dandi_instance_id):
         for name, rec in server_info.get("services", {}).items()
     }
     if services.get("api"):
-        return dandi_instance(
+        return DandiInstance(
             gui=services.get("webui"),
             redirector=redirector_url,
             api=services.get("api"),
@@ -616,25 +585,6 @@ def get_instance(dandi_instance_id):
         )
 
 
-def try_multiple(ntrials, retry, base):
-    """
-    ``try_multiple(ntrials, retry, base)(f, *args, **kwargs)`` calls ``f``
-    multiple times until it succeeds, with exponentially increasing delay
-    between calls
-    """
-    # `retry` must be an exception type, a tuple of exception types, or a valid
-    # `retry` argument to tenacity.
-    if isinstance(retry, (type, tuple)):
-        retry = tenacity.retry_if_exception_type(retry)
-    return tenacity.Retrying(
-        wait=tenacity.wait_exponential(exp_base=base, multiplier=base),
-        retry=retry,
-        stop=tenacity.stop_after_attempt(ntrials),
-        before_sleep=tenacity.before_sleep_log(lgr, logging.WARNING),
-        reraise=True,
-    )
-
-
 def is_url(s):
     """Very primitive url detection for now
 
@@ -729,3 +679,32 @@ def get_mime_type(filename: str, strict: bool = False) -> str:
         # return mtype + '+gzip'
     else:
         return "application/x-" + encoding
+
+
+def check_dandi_version():
+    if os.environ.get("DANDI_NO_ET"):
+        return
+    try:
+        import etelemetry
+
+        try:
+            etelemetry.check_available_version(
+                "dandi/dandi-cli", __version__, lgr=lgr, raise_exception=True
+            )
+        except etelemetry.client.BadVersionError:
+            # note: SystemExit is based of BaseException, so is not Exception
+            raise SystemExit(
+                "DANDI CLI has detected that you are using a version that is known to "
+                "contain bugs, is incompatible with our current data archive, or has "
+                "other significant performance limitations. "
+                "To continue using DANDI CLI, please upgrade your dandi client to a newer "
+                "version (e.g., using pip install --upgrade dandi if you installed using pip). "
+                "If you have any issues, please contact the DANDI "
+                "helpdesk at https://github.com/dandi/helpdesk/issues/new/choose ."
+            )
+    except Exception as exc:
+        lgr.warning(
+            "Failed to check for a more recent version available with etelemetry: %s",
+            exc,
+        )
+    os.environ["DANDI_NO_ET"] = "1"
